;*********************************************************************
;*                SEGGER MICROCONTROLLER SYSTEME GmbH                *
;*        Solutions for real time microcontroller applications       *
;*********************************************************************
;*                                                                   *
;*      (C) 2006    SEGGER Microcontroller Systeme GmbH              *
;*                  www.segger.com                                   *
;*                                                                   *
;*********************************************************************
;
;---------------------------------------------------------------------
;File   : init_AT91SAM7S.s
;Purpose: startup file for AT91SAM7S CPUs and embOS
;-------- END-OF-HEADER ----------------------------------------------
;

; This module performs ROM/RAM remapping (if required), initializes stack 
; pointers and interrupts for each mode, and finally branches to __main in 
; the C library (which eventually calls main()).
;
; On reset, the ARM core starts up in Supervisor (SVC) mode, in ARM state, 
; with IRQ and FIQ disabled.


        PRESERVE8        
        AREA    Init, CODE, READONLY
; --- ensure no functions that use semihosting SWIs are linked in from the C library
                IMPORT __use_no_semihosting_swi
      
; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs

Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled


        ENTRY

        EXPORT  Reset_Handler
        IMPORT  __main
        IMPORT  __low_level_init
        IMPORT  __program_start
        IMPORT  __vector_end
        
Reset_Handler

; --- Initialize stack pointer registers

; Enter each mode in turn and set up the stack pointer

        IMPORT  ||Image$$CSTACK$$ZI$$Limit||       ; defined in scatter file
        IMPORT  ||Image$$IRQ_STACK$$ZI$$Limit||    ; defined in scatter file

        msr     CPSR_c, #Mode_IRQ:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     sp, =||Image$$IRQ_STACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8

        msr     CPSR_c, #Mode_SVC:OR:I_Bit:OR:F_Bit ; No interrupts
        ldr     sp, =||Image$$CSTACK$$ZI$$Limit||
        and     sp, sp, #0xFFFFFFF8
         
        IF :DEF: REMAP
        mov     r0, #0
        ldr     r1, [r0]
        add     r2, r1, #1
        str     r2, [r0]
        ldr     r3, [r0]
        cmp     r2, r3
        bne     remap_ram
        str     r1, [r0]
        b       copy_vector
        
remap_ram
        ldr     r0, =0xFFFFFF00
        mov     r1, #0x01
        str     r1, [r0]        
        ENDIF
        
copy_vector        
        ldr     r0, = __program_start
        ldr     r1, = 0x00000        
        ldr     r2, = __vector_end
        sub     r2, r2, r0
        mov     r2, r2 , lsr #2
copy_vector_loop
        ldr     r3,[r0], #+4
        str     r3,[R1], #+4
        subs    r2,r2,#+0x1
        bne     copy_vector_loop

; --- Initialize critical IO devices
        ; Call  __low_level_init
        bl      __low_level_init        

; --- Now enter the C code
        b       __main   ; note use B not BL, because an application will never return this way

        END


