/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File   : RTOSInit_SoCLitePlus.c
Purpose: Initializes and handles the hardware for embOS as far
         as required by embOS.
         Feel free to modify this file acc. to your target system.
-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configurable defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/

#ifndef   OS_FSYS
  #define OS_FSYS  50000000
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER OS_FSYS
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  OS_FSYS
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ 1000
#endif

#define   _OS_TIMER_PRESCALE  1    /* prescaler for timer frequency */

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*       and supports UART0 and UART1
*/

#ifndef   OS_UART
  #define OS_UART 0
#endif    

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400
#endif

#define _OS_UART_USED (OS_UART == 0)

/****** Define behaviour of undefined interrupt handling ************/

#ifndef   OS_IGNORE_UNDEFINED_INTERRUPT
  #define OS_IGNORE_UNDEFINED_INTERRUPT 0
#endif

/*********************************************************************
*
*       Local defines
*
**********************************************************************
*/

/*********************************************************************
*
*       Reference to external functions used in RTOSInit_SoCLitePlus.c
*/

#ifdef __cplusplus
  extern "C" {
#endif
void sdram_init (void);
void sram_init (void);
#ifdef __cplusplus
  } 
#endif

/*********************************************************************
*
*       sfrs used in RTOSInit_SoCLitePlus.c
*/

/********************************************
*   MultiPort Memory Controller Registers 
*********************************************
*/
#define __MPMC_BASE         0x54003000 
#define __MPMCCtrl          *((volatile OS_U32*)(__MPMC_BASE + 0x00))    /* General Control */
#define __MPMCStatus        *((volatile OS_U32*)(__MPMC_BASE + 0x04))    /* General Status */
#define __MPMCConf          *((volatile OS_U32*)(__MPMC_BASE + 0x08))    /* General Configuration */
#define __MPMCDynCtrl       *((volatile OS_U32*)(__MPMC_BASE + 0x20))    /* Dynamic Control */
#define __MPMCDynRef        *((volatile OS_U32*)(__MPMC_BASE + 0x24))    /* Dynamic Refresh Timer */
#define __MPMCDynReadConfig *((volatile OS_U32*)(__MPMC_BASE + 0x28))    /* Dynamic Read Configuration - Strategy */
#define __MPMCDynRP         *((volatile OS_U32*)(__MPMC_BASE + 0x30))    /* Dynamic Precharge Period */
#define __MPMCDynRAS        *((volatile OS_U32*)(__MPMC_BASE + 0x34))    /* Dynamic RAS */
#define __MPMCDynSREX       *((volatile OS_U32*)(__MPMC_BASE + 0x38))    /* Dynamic Self-Refresh Time */
#define __MPMCDynAPR        *((volatile OS_U32*)(__MPMC_BASE + 0x3C))    /* Dynamic Last-Data-Out Time */
#define __MPMCDynDAL        *((volatile OS_U32*)(__MPMC_BASE + 0x40))    /* Dynamic Dat-In Time */
#define __MPMCDynWR         *((volatile OS_U32*)(__MPMC_BASE + 0x44))    /* Dynamic Write Recovery Time */
#define __MPMCDynRC         *((volatile OS_U32*)(__MPMC_BASE + 0x48))    /* Dynamic Active-to-Active */
#define __MPMCDynRFC        *((volatile OS_U32*)(__MPMC_BASE + 0x4C))    /* Dynamic Auto-Refresh */
#define __MPMCDynXSR        *((volatile OS_U32*)(__MPMC_BASE + 0x50))    /* Dynamic Exit Self-Refresh */
#define __MPMCDynRRD        *((volatile OS_U32*)(__MPMC_BASE + 0x54))    /* Dynamic Active Bank A-to-B */
#define __MPMCDynMRD        *((volatile OS_U32*)(__MPMC_BASE + 0x58))    /* Dynamic Load Mode */
#define __MPMCStatExtWait   *((volatile OS_U32*)(__MPMC_BASE + 0x80))    /* Static Extended Wait */
#define __MPMCDynCfg0       *((volatile OS_U32*)(__MPMC_BASE + 0x100))   /* Dynamic Configuration CS0 */
#define __MPMCDynRasCas0    *((volatile OS_U32*)(__MPMC_BASE + 0x104))   /* Dynamic RAS-CAS Delay CS0 */
#define __MPMCDynCfg1       *((volatile OS_U32*)(__MPMC_BASE + 0x120))   /* Dynamic Configuration CS1 */
#define __MPMCDynRasCas1    *((volatile OS_U32*)(__MPMC_BASE + 0x124))   /* Dynamic RAS-CAS Delay CS1 */
#define __MPMCDynCfg2       *((volatile OS_U32*)(__MPMC_BASE + 0x140))   /* Dynamic Configuration CS2 */
#define __MPMCDynRasCas2    *((volatile OS_U32*)(__MPMC_BASE + 0x144))   /* Dynamic RAS-CAS Delay CS2 */
#define __MPMCDynCfg3       *((volatile OS_U32*)(__MPMC_BASE + 0x160))   /* Dynamic Configuration CS3 */
#define __MPMCDynRasCas3    *((volatile OS_U32*)(__MPMC_BASE + 0x164))   /* Dynamic RAS-CAS Delay CS3 */
#define __MPMCStatCfg0      *((volatile OS_U32*)(__MPMC_BASE + 0x200))   /* Static Configuration CS0 */
#define __MPMCStatWEWait0   *((volatile OS_U32*)(__MPMC_BASE + 0x204))   /* Static Write-Enable Delay CS0 */
#define __MPMCStatOEWait0   *((volatile OS_U32*)(__MPMC_BASE + 0x208))   /* Static Output-Enable Delay CS0 */
#define __MPMCStatRDWait0   *((volatile OS_U32*)(__MPMC_BASE + 0x20C))   /* Static Read Delay CS0 */
#define __MPMCStatPGWait0   *((volatile OS_U32*)(__MPMC_BASE + 0x210))   /* Static Page-Mode Read CS0 */
#define __MPMCStatWRWait0   *((volatile OS_U32*)(__MPMC_BASE + 0x214))   /* Static Write Delay CS0 */
#define __MPMCStatTRWait0   *((volatile OS_U32*)(__MPMC_BASE + 0x218))   /* Static Turn-Round Delay CS0 */
#define __MPMCStatCfg1      *((volatile OS_U32*)(__MPMC_BASE + 0x220))   /* Static Configuration CS1 */
#define __MPMCStatWEWait1   *((volatile OS_U32*)(__MPMC_BASE + 0x224))   /* Static Write-Enable Delay CS1 */
#define __MPMCStatOEWait1   *((volatile OS_U32*)(__MPMC_BASE + 0x228))   /* Static Output-Enable Delay CS1 */
#define __MPMCStatRDWait1   *((volatile OS_U32*)(__MPMC_BASE + 0x22C))   /* Static Read Delay CS1 */
#define __MPMCStatPGWait1   *((volatile OS_U32*)(__MPMC_BASE + 0x230))   /* Static Page-Mode Read CS1 */
#define __MPMCStatWRWait1   *((volatile OS_U32*)(__MPMC_BASE + 0x234))   /* Static Write Delay CS1 */
#define __MPMCStatTRWait1   *((volatile OS_U32*)(__MPMC_BASE + 0x238))   /* Static Turn-Round Delay CS1 */
#define __MPMCStatCfg2      *((volatile OS_U32*)(__MPMC_BASE + 0x240))   /* Static Configuration CS2 */
#define __MPMCStatWEWait2   *((volatile OS_U32*)(__MPMC_BASE + 0x244))   /* Static Write-Enable Delay CS2 */
#define __MPMCStatOEWait2   *((volatile OS_U32*)(__MPMC_BASE + 0x248))   /* Static Output-Enable Delay CS2 */
#define __MPMCStatRDWait2   *((volatile OS_U32*)(__MPMC_BASE + 0x24C))   /* Static Read Delay CS2 */
#define __MPMCStatPGWait2   *((volatile OS_U32*)(__MPMC_BASE + 0x250))   /* Static Page-Mode Read CS2 */
#define __MPMCStatWRWait2   *((volatile OS_U32*)(__MPMC_BASE + 0x254))   /* Static Write Delay CS2 */
#define __MPMCStatTRWait2   *((volatile OS_U32*)(__MPMC_BASE + 0x258))   /* Static Turn-Round Delay CS2 */
#define __MPMCStatCfg3      *((volatile OS_U32*)(__MPMC_BASE + 0x260))   /* Static Configuration CS3 */
#define __MPMCStatWEWait3   *((volatile OS_U32*)(__MPMC_BASE + 0x264))   /* Static Write-Enable Delay CS3 */
#define __MPMCStatOEWait3   *((volatile OS_U32*)(__MPMC_BASE + 0x268))   /* Static Output-Enable Delay CS3 */
#define __MPMCStatRDWait3   *((volatile OS_U32*)(__MPMC_BASE + 0x26C))   /* Static Read Delay CS3 */
#define __MPMCStatPGWait3   *((volatile OS_U32*)(__MPMC_BASE + 0x270))   /* Static Page-Mode Read CS3 */
#define __MPMCStatWRWait3   *((volatile OS_U32*)(__MPMC_BASE + 0x274))   /* Static Write Delay CS3 */
#define __MPMCStatTRWait3   *((volatile OS_U32*)(__MPMC_BASE + 0x278))   /* Static Turn-Round Delay CS3 */
#define __MPMCCITCR         *((volatile OS_U32*)(__MPMC_BASE + 0xF00))   /* Test Control */
#define __MPMCITIP          *((volatile OS_U32*)(__MPMC_BASE + 0xF20))   /* Test Input */
#define __MPMCITOP          *((volatile OS_U32*)(__MPMC_BASE + 0xF40))   /* Test Output Register */
#define __MPMCPerlD4        *((volatile OS_U32*)(__MPMC_BASE + 0xFD0))   /* Peripheral ID bits 39:32 */
#define __MPMCPerlD5        *((volatile OS_U32*)(__MPMC_BASE + 0xFD4))   /* Peripheral ID (reserved) */
#define __MPMCPerlD6        *((volatile OS_U32*)(__MPMC_BASE + 0xFD8))   /* Peripheral ID (reserved) */
#define __MPMCPerlD7        *((volatile OS_U32*)(__MPMC_BASE + 0xFDC))   /* Peripheral ID (reserved) */
#define __MPMCPerlD0        *((volatile OS_U32*)(__MPMC_BASE + 0xFE0))   /* Peripheral ID bits 7:0 */
#define __MPMCPerlD1        *((volatile OS_U32*)(__MPMC_BASE + 0xFE4))   /* Peripheral ID bits 15:8 */
#define __MPMCPerlD2        *((volatile OS_U32*)(__MPMC_BASE + 0xFE8))   /* Peripheral ID bits 23:16 */
#define __MPMCPerlD3        *((volatile OS_U32*)(__MPMC_BASE + 0xFEC))   /* Peripheral ID bits 31:24 */
#define __MPMCPCID0         *((volatile OS_U32*)(__MPMC_BASE + 0xFF0))   /* PrimeCell ID bits 7:0 */      
#define __MPMCPCID1         *((volatile OS_U32*)(__MPMC_BASE + 0xFF4))   /* PrimeCell ID bits 15:8 */
#define __MPMCPCID2         *((volatile OS_U32*)(__MPMC_BASE + 0xFF8))   /* PrimeCell ID bits 23:16 */ 
#define __MPMCPCID3         *((volatile OS_U32*)(__MPMC_BASE + 0xFFC))   /* PrimeCell ID bits 31:24 */ 

/********************************************
*   Write Protection Registers 
*********************************************
*/
#define __WP_BASE     0x54004000 
#define __WPC       *((volatile OS_U32*)(__WP_BASE + 0x00))     /* Write Protect Control */
#define __WPS       *((volatile OS_U32*)(__WP_BASE + 0x04))     /* Write Protect Status */

/********************************************
  Interrupt Controller Registers 
*********************************************
*/
#define __INTC_BASE   0x54006400 
#define __INTC0     *((volatile OS_U32*)(__INTC_BASE + 0x00))   /* Interrupt Config. 0  */
#define __INTC1     *((volatile OS_U32*)(__INTC_BASE + 0x04))   /* Interrupt Config. 1  */
#define __INTC2     *((volatile OS_U32*)(__INTC_BASE + 0x08))   /* Interrupt Config. 2  */
#define __INTC3     *((volatile OS_U32*)(__INTC_BASE + 0x0C))   /* Interrupt Config. 3  */
#define __INTC4     *((volatile OS_U32*)(__INTC_BASE + 0x10))   /* Interrupt Config. 4  */
#define __INTC5     *((volatile OS_U32*)(__INTC_BASE + 0x14))   /* Interrupt Config. 5  */
#define __INTC6     *((volatile OS_U32*)(__INTC_BASE + 0x18))   /* Interrupt Config. 6  */
#define __INTC7     *((volatile OS_U32*)(__INTC_BASE + 0x1C))   /* Interrupt Config. 7  */
#define __INTSTA    *((volatile OS_U32*)(__INTC_BASE + 0x20))   /* INTC Status          */
#define __INTSRC    *((volatile OS_U8 *)(__INTC_BASE + 0x24))   /* Interrupt Source No. */

/********************************************
*   UART Registers 
*********************************************
*/
#define __UART_BASE   0x54006800 
#define __SRXB      *((volatile OS_U8 *)(__UART_BASE + 0x00))   /* UART Receive Buffer  */
#define __STXB      *((volatile OS_U8 *)(__UART_BASE + 0x04))   /* UART Transmit Buffer */
#define __SSTA      *((volatile OS_U8 *)(__UART_BASE + 0x08))   /* UART Status  */
#define __SCTR      *((volatile OS_U32*)(__UART_BASE + 0x0C))   /* UART Control */

/********************************************
*   TIMER Registers 
*********************************************
*/
#define __TIMER_BASE    0x54006C00 
#define __TMV       *((volatile OS_U32*)(__TIMER_BASE + 0x00))  /* Timer Counter */
#define __TML       *((volatile OS_U32*)(__TIMER_BASE + 0x04))  /* Timer Load    */
#define __TMC       *((volatile OS_U8 *)(__TIMER_BASE + 0x08))  /* Timer Control */

/********************************************
*   Watchdog Register 
*********************************************
*/
#define __WD_BASE     0x54007000 
#define __WDCTRL      *((volatile OS_U16*)(__WD_BASE + 0x00))   /* Watchdog Timer Control */

/********************************************
*   Remap Register 
*********************************************
*/
#define __RMP_BASE    0x54007400 
#define __RMP       *((volatile OS_U8*)(__RMP_BASE + 0x00))     /* Remap Control */


/********************************************
*   APB Bridge Register 
*********************************************
*/
#define __APB_BASE    0x54007800 
#define __APBW      *((volatile OS_U8*)(__APB_BASE + 0x00))     /* APB Wait Control */


/*********************************************************************
*
*       Defines non-configurable
*
**********************************************************************
*/
#define NUM_INT_SOURCES      32
#define _INT_PRIORITY_MASK 0x07

#define _TIMER_RELOAD      (OS_PCLK_TIMER/_OS_TIMER_PRESCALE/OS_TICK_FREQ)
#define _OS_TIMER_ID       29
#define _OS_TIMER_PRIO     0x07           // lowest priority

#define _OS_UART_TX_ID     31
#define _OS_UART_RX_ID     30
#define _OS_UART_TX_PRIO   0x06           // lowest priority
#define _OS_UART_RX_PRIO   0x06           // lowest priority

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static volatile OS_U32 _Dummy;
static OS_ISR_HANDLER* _apOS_ISRHandler[NUM_INT_SOURCES];

/*********************************************************************
*
*       Static code
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_Tick interrupt Handler
*/
static void _OS_ISR_Tick(void) {
  OS_HandleTick();                           // Call embOS tick handler
}

#if _OS_UART_USED
/*********************************************************************
*
*       OS_UART receive interrupt Handler
*/

#define _RX_DATA_VALID    (1 << 0)
#define _RX_DATA_ERROR    (1 << 1)

static void _OS_ISR_UARTRX(void) {
  OS_U8 Status;
  OS_U8 Data;
  Status = __SSTA;
  do {
    if (Status & _RX_DATA_ERROR) {
      __SSTA |= _RX_DATA_ERROR;   // Clear error
    } else {
      Data  = __SRXB;
      OS_OnRx(Data);
    }
    Status = __SSTA;
 } while ((Status & _RX_DATA_VALID) == _RX_DATA_VALID);
}

/*********************************************************************
*
*       OS_UART transmit interrupt Handler
*/
static void _OS_ISR_UARTTX(void) {
  OS_OnTx();
}

#endif /* _OS_UART_USED */

/*********************************************************************
*
*       Public code
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for embOS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
//  sdram_init();
//  sram_init();
  __TML = _TIMER_RELOAD;
  __TMC = 1;                                             /* Enable timer1 */
  __TMV = _TIMER_RELOAD;
  OS_ARM_InstallISRHandler(_OS_TIMER_ID, &_OS_ISR_Tick); // Timer/counter interrupt vector.
  OS_ARM_ISRSetPrio(_OS_TIMER_ID, _OS_TIMER_PRIO);       // Timer/counter interrupt priority.
  OS_ARM_EnableISR(_OS_TIMER_ID);                        // Enable timer/counter 0 interrupt.
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) { /* Idle loop: No task is ready to exec */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via embOSView or high 
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/

#define _HW_TIMER                      __TMV
#define _HW_TIMER_INTERRUPT_PENDING() (__INTSTA & (1 << _OS_TIMER_ID))

OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 TimerCnt;
  OS_U32 Time;
  
  TimerCnt = _TIMER_RELOAD - _HW_TIMER;
  Time     = OS_GetTime32();
  if (_HW_TIMER_INTERRUPT_PENDING()) {
    TimerCnt = _TIMER_RELOAD - _HW_TIMER;
    Time++;
  }
  return (_TIMER_RELOAD) * Time + TimerCnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= _OS_TIMER_PRESCALE;    /* eliminate effect of prescaler */
  return Cycles/(OS_PCLK_TIMER/1000000);
}


/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/


/*********************************************************************
*
*       Final check of configuration
*
**********************************************************************
*/

#ifndef _OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

#if _OS_UART_USED
/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for embOSView
*/
#define _BAUDDIVIDE ((OS_PCLK_UART + OS_BAUDRATE * 8L)/(OS_BAUDRATE * 8L) - 1)

void OS_COM_Init(void) {
  __SCTR  = _BAUDDIVIDE;         // Set baudrate
  OS_ARM_InstallISRHandler(_OS_UART_TX_ID, &_OS_ISR_UARTTX); // Timer/counter interrupt vector.
  OS_ARM_ISRSetPrio(_OS_UART_TX_ID, _OS_UART_TX_PRIO);       // Timer/counter interrupt priority.
  OS_ARM_EnableISR(_OS_UART_TX_ID);                          // Enable timer/counter 0 interrupt.

  OS_ARM_InstallISRHandler(_OS_UART_RX_ID, &_OS_ISR_UARTRX); // Timer/counter interrupt vector.
  OS_ARM_ISRSetPrio(_OS_UART_RX_ID, _OS_UART_RX_PRIO);       // Timer/counter interrupt priority.
  OS_ARM_EnableISR(_OS_UART_RX_ID);                          // Enable timer/counter 0 interrupt.
  __SCTR |= (1 << 14)            // Enable Receive
         |  (1 << 15)            // Enable Transmit
         ;
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  __STXB = c;
}

#else 
void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);
  OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
}
#endif  /* _OS_UART_USED */

/*********************************************************************
*
*       embOS interrupt handler and ISR specific functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_ISR_Undefined
*
*       Is called when an uninstalled interrupt was detected
*       As interrupt pending condition of peripherals has to be reset,
*       program will not continue when interrupt is ignored.
*/
static void _OS_ISR_Undefined(int Index) {
  #if (OS_IGNORE_UNDEFINED_INTERRUPT == 0)
    _Dummy = Index;
    /* You may set a breakpoint here to detect undefined interrupts */
    while (_Dummy < NUM_INT_SOURCES) {
    }
  #endif
} 

/*********************************************************************
*
*       OS_irq_handler
*
*       Detect reason for IRQ and call correspondig service routine.
*       OS_irq_handler is called from embOS OS_IRQ_SERVICE function
*       found in RTOSVect.asm
*/
void OS_irq_handler(void) {
  OS_ISR_HANDLER* pISR;
  int ISRIndex;

  ISRIndex = __INTSRC;                        // Perform a dummy vector read to update CICR
  pISR = _apOS_ISRHandler[ISRIndex];          // Read interrupt vector
  OS_EnterNestableInterrupt();                // Now interrupts may be reenabled. If nesting should be allowed
  if (pISR != NULL) {
    pISR();                                   // Call installed interrupt service routine
  } else {
    _OS_ISR_Undefined(ISRIndex);
  }
  OS_DI();                                    // Disable interrupts and unlock
  __INTSTA = (1 << ISRIndex);                 // Clear current interrupt pending bit, reset EIC
  OS_LeaveNestableInterrupt();                // Replace by OS_LeaveInterrupt(), when nesting was disabled 
}

/*********************************************************************
*
*       OS_ARM_InstallISRHandler
*
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler (int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
  OS_ISR_HANDLER*  pOldHandler;
  
  pOldHandler = NULL;
  OS_DI();
  if (ISRIndex < NUM_INT_SOURCES) {
    pOldHandler                = _apOS_ISRHandler[ISRIndex];
    _apOS_ISRHandler[ISRIndex] = pISRHandler;
  }   
  OS_RestoreI();
  return pOldHandler;    
}

/*********************************************************************
*
*       OS_ARM_EnableISR
*
*/
void OS_ARM_EnableISR(int ISRIndex) {
  OS_U32 * pISROff;
  OS_U32   Off;

  OS_DI();
  if (ISRIndex < NUM_INT_SOURCES) {
    pISROff  = (OS_U32*)&__INTC0;
    pISROff += (ISRIndex >> 2);
    Off      = ISRIndex & 3;
    *pISROff |= (1 << ((Off << 3) + 6));
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_DisableISR
*
*/
void OS_ARM_DisableISR(int ISRIndex) {
  OS_U32 * pISROff;
  OS_U32   Off;

  OS_DI();
  if (ISRIndex < NUM_INT_SOURCES) {
    pISROff  = (OS_U32*)&__INTC0;
    pISROff += (ISRIndex >> 2);
    Off      = ISRIndex & 3;
    *pISROff &= ~(1 << ((Off << 3) + 6));
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio
*
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U32* pPrio;
  int     OldPrio;
  int     Off;

  OS_DI();
  pPrio     = (OS_U32*)&__INTC0;
  pPrio    += (ISRIndex >> 2);
  Off       = ISRIndex & 3;
  OldPrio   = *pPrio;
  OldPrio >>= (Off << 3);
  *pPrio   |= ((OldPrio & ~_INT_PRIORITY_MASK) | (Prio & _INT_PRIORITY_MASK)) << (Off << 3);
  OS_RestoreI();
  return OldPrio & _INT_PRIORITY_MASK;
}

/*****  EOF  ********************************************************/

