/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

-------------------------------------------------------------------------
        File Information                                                   
  -------------------------------------------------------------------------
        Name    : SDRAM_Config.h                                                   
        Type    : C Programming Language File                                
        Date    : 2005 May 4th                                          
        CPU     : ARM7TDMI-S  (SoCLite+)                
        IDE     : IAR EWARM V4.31A
        Author  : NEC Electronics (Europe) GmbH                                            
  -------------------------------------------------------------------------
        About   : Header Function   (SDRAM Header)                             
  -------------------------------------------------------------------------
        Copyright (C) NEC Corporation 2005                                 
  -------------------------------------------------------------------------
    

 ****************************************************************************
 ****************************************************************************/

#ifndef SDRAM_H                                 /* Avoid multiple inclusion */
#define SDRAM_H

/******************************************************************************
 ******************************************************************************

          SDRAM Definition / Configuration
 
 ******************************************************************************
 ******************************************************************************/

/* Only SDRAM in 16-bit wide configuration is used on Start it! development board */

/*********************************************\
  Elpida SDRAM Settings (DS2516APTA-75)
\*********************************************/


/* fsys = 50 MHz */
/* HCLK to MPMCCLK ratio is 1:1 */


/* Definition of SDRAM operation commands */

#define  MPMC_NOP     (0x00000183)    /* no operation */
#define  MPMC_PALL    (0x00000103)    /* precharge all operation */
#define  MPMC_MODE    (0x00000083)    /* mode command operation */
#define  MPMC_NORMAL  (0x00000003)    /* normal command operation */

/* Definition of SDRAM operation parameters */

#define  WAIT_100       (5000)        /* 100 us wait at 50 MHz */
#define  WAIT_200       (10000)       /* 200 us wait at 50 MHz */
#define  ELPIDA_REFCNT  (256)         /* issue 8 refresh cycles for power-up initialization */
#define  ELPIDA_REFINIT (1)           /* issue refresh cycles every 32 HCLK ticks */
#define  ELPIDA_REF     (8)           /* refresh period between SDRAM refresh cycles (8192 cycles within 64ms) */
#define  ELPIDA_RASCAS  (0x0303)      /* CAS latency is 3; RAS latency is 3 */
#define  ELPIDA_RP      (3)           /* precharge command period; min: 20 ns  */
#define  ELPIDA_RAS     (6)           /* active to precharge command period; min: 45 ns  */
#define  ELPIDA_SREX    (2)           /* self-refresh exit time; min: 8 ns  */
#define  ELPIDA_APR     (2)           /* last-data-out to active command time; min: 8 ns  */
#define  ELPIDA_DAL     (5)           /* data-in to active command time; min: 2CLK + 20 ns  */
#define  ELPIDA_WR      (2)           /* write recovery time; min: 2 MPMCCLK or 15 ns  */
#define  ELPIDA_RC      (9)           /* active to active command period; min: 9 MPMCCLK or 67.5 ns  */
#define  ELPIDA_RFC     (9)           /* active to active command period; min: 9 MPMCCLK or 67.5 ns  */
#define  ELPIDA_XSR     (9)           /* self-refresh exit to active command time; min: 9 MPMCCLK or 67.5 ns  */
#define  ELPIDA_RRD     (2)           /* bank A to bank B command latency; min: 2 MPMCCLK or 15 ns  */
#define  ELPIDA_MRD     (2)           /* load mode register to active command time; min: 2 MPMCCLK */
#define  ELPIDA_RD      (1)           /* command delayed strategy */

/*  MPMC Dynamic Memory Configuration Registers 3-0 */

/*
----------------------------------------------------------------------------------------------
Bits    Name                Type    Description
----------------------------------------------------------------------------------------------
[31:21]   -           -     Reserved, read undefined, do not modify.
----------------------------------------------------------------------------------------------
[20]    Write protect (P)   R/w     0 = writes not protected (reset value on nPOR)
                                    1 = write protected.
----------------------------------------------------------------------------------------------
[19]    Buffer enable(B)    R/w     0 = buffer disabled (reset value on nPOR)
                                    1 = buffer enabled (a)
                            for accesses to this chip select
----------------------------------------------------------------------------------------------
[18:15]   -           -     Reserved, read undefined, do not modify.
----------------------------------------------------------------------------------------------
[14]    Address mapping(AM) R/w     See Table 3-22 on page 3-26.
                                    0 = reset value on nPOR.
----------------------------------------------------------------------------------------------
[13]    -           -     Reserved, read undefined, do not modify.
----------------------------------------------------------------------------------------------
[12:7]  Address mapping(AM) R/w     See Table 3-22 on page 3-26.
                                    000000 = reset value on nPOR.
----------------------------------------------------------------------------------------------
[6:5]   -           -     Reserved, read undefined, do not modify.
----------------------------------------------------------------------------------------------
[4:3]   Memory device(MD)   R/w     00 = SDRAM (reset value on nPOR)
                                    01 = low-power SDRAM
                                    10 = Micron SyncFlash
                                    11 = reserved.
----------------------------------------------------------------------------------------------
[2:0] -             -     Reserved, read undefined, do not modify.
----------------------------------------------------------------------------------------------
                           3  .    .   2.    .    . 1  .
                          1098.7654.3210.9876.5432.1098.7654.3210
------------------------------.----.----.----.----.----.----.----
         ELPIDA_MASK    0b1111.1111.1110.0111.1010.0000.0110.0111
32-bit   ELPIDA_TYPE    0b0000.0000.0000.0000.0100.0110.1000.0000
16-bit   ELPIDA_TYPE    0b0000.0000.0000.0000.0000.0110.1000.0000
------------------------------.----.----.----.----.----.----.----
*/

#define  ELPIDA_MASK    (0xffe7a067)      /* Mask for initialization */


/*
  Select of 32-bit or 16-bit external memory bus
  bit(14):  0 = 16-bit external bus
        1 = 32-bit external bus
  bit(12):  0 = address mapping row-bank-column (RBC)
        1 = address mapping bank-row-column (BRC) 
*/

/* use 16-bit external SDRAM I/F */
#define  ELPIDA_TYPE      (0x00000680)    /* RBC mapping used, buffer disabled */
#define  ELPIDA_OPERATION (0x00080680)    /* RBC mapping used, buffer enabled */  
#define  ELPIDA_MODE      (0x33)          /* burst r/w; CAS=3; sequential burst; burst lenght 8 */
#define  ELPIDA_SHIFT     (0x1000)        /* shift left by 11 to calculate address offset */


#define  ELPIDA_ADDR  (ELPIDA_MODE * ELPIDA_SHIFT)    /* Offset address to write Mode register of Elpida SDRAM */

/*********************************************************************
*
*       SDRAM_Init.c
*
**********************************************************************
*/

#ifndef __IAR_SYSTEMS_ICC__
  #define __interwork
#endif

__interwork void SDRAM_Init(void);                  /* SDRAM_Init.c */

/********************************************************************/

#endif /* Avoid multiple inclusion */

/****** EOF *********************************************************/
