/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
        File Information
----------------------------------------------------------------------
        Name    : SDRAM-Init.c
        Type    : C Programming Language File
        Date    : 2005 May 4th
        CPU     : ARM7TDMI-S  (SoCLite+)
        IDE     : IAR EWARM V4.31A
        Author  : NEC Electronics (Europe) GmbH
----------------------------------------------------------------------
        About   : SDRAM Function    (SDRAM Intialisation)
----------------------------------------------------------------------
        Copyright (C) NEC Corporation 2005
----------------------------------------------------------------------

**********************************************************************
*********************************************************************/

#include "SoCLitePlus.h"
#include "SDRAM_Config.h"

#define DCS5_BASE     (0x20000000)   /* dynamic CS5 basic address */
#define ELPIDA_READ   (*((volatile unsigned int*) (DCS5_BASE + ELPIDA_ADDR)))   /* Program address for Elpida mode register */


/*********************************************************************
*
*       SDRAM_Init()
*
*       Initialize memory controller for SDRAM
*
* NOTES:
*  (1) ICODE is the same segment as cstartup. By placing SDRAM_Init
*      in the same segment, we make sure it can be reached with BL.
*
*  (2) All MPMC registers have to be accessed with read-modify write operation
*      because they contain reserved undefined bits, which shall not be modified
*/
#ifndef   INIT_SDRAM
  #define INIT_SDRAM 1       /* SDRAM is initialized per default, but must not be */
#endif                       /* initialized when program runs from SDRAM */

#ifdef __IAR_SYSTEMS_ICC__
#pragma location="ICODE"     /* (Note 1) */
#endif

__interwork void SDRAM_Init(void) {
  #if INIT_SDRAM
    int iCnt;
    volatile int read;       /* declaration as volatile ensures that register is read and suppresses a warning */

    /* Basic Settings */

    /* Set MPMC Control register */
    MPMCCtrl = ( MPMCCtrl & 0xfffffff8 ) | 0x3 ;      /* read-modify write operation */
    iCnt = 0; while ( iCnt < WAIT_100 ) { iCnt++; }

    /* Set MPMC Configuration register */
    MPMCConf = ( MPMCConf & 0xfffffefe ) | 0x0 ;      /* read-modify write operation */
    iCnt = 0; while ( iCnt < WAIT_100 ) { iCnt++; }

    /* Set MPMC Read Configuration register */
    MPMCDynReadConfig = ( MPMCDynReadConfig & 0xfffffffc ) | ELPIDA_RD ;  /* read-modify write operation */
    iCnt = 0; while ( iCnt < WAIT_100 ) { iCnt++; }

    /* Start of SDRAM Initialisation */

    /* Wait for 100 us to stabilize the clocks */
    iCnt = 0;
    while ( iCnt < WAIT_100 ) { iCnt++; }

    /* Issue NOP command */
    MPMCDynCtrl = ( MPMCDynCtrl & 0xffff1e58 ) | MPMC_NOP ;     /* read-modify write operation */

    /* Wait for 200 us to stabilize the clocks */
    iCnt = 0; while ( iCnt < WAIT_200 ) { iCnt++; }

    /* Issue PALL command */
    MPMCDynCtrl = ( MPMCDynCtrl & 0xffff1e58 ) | MPMC_PALL ;

    /* Issue auto refresh cycle every 32 HCLK ticks */
    MPMCDynRef = ( MPMCDynRef & 0xfffff800 ) | ELPIDA_REFINIT ;

    /* Wait until 8 refresh cycles have occured: 8 x 32 HCLK ticks = 256 */
    iCnt = 0; while (iCnt < ELPIDA_REFCNT) { iCnt++; }

    /* Set operation values into MPMC registers */
    MPMCDynRef      = ( MPMCDynRef & 0xfffff800 ) | ELPIDA_REF ;

    MPMCDynRP       = ( MPMCDynRP   & 0xfffffff0 ) | ELPIDA_RP ;
    MPMCDynRAS      = ( MPMCDynRAS  & 0xfffffff0 ) | ELPIDA_RAS ;
    MPMCDynSREX     = ( MPMCDynSREX & 0xfffffff0 ) | ELPIDA_SREX ;
    MPMCDynAPR      = ( MPMCDynAPR  & 0xfffffff0 ) | ELPIDA_APR ;
    MPMCDynDAL      = ( MPMCDynDAL  & 0xfffffff0 ) | ELPIDA_DAL ;
    MPMCDynWR       = ( MPMCDynWR   & 0xfffffff0 ) | ELPIDA_WR ;

    MPMCDynRC       = ( MPMCDynRC  & 0xffffffe0 ) | ELPIDA_RC ;
    MPMCDynRFC      = ( MPMCDynRFC & 0xffffffe0 ) | ELPIDA_RFC ;
    MPMCDynXSR      = ( MPMCDynXSR & 0xffffffe0 ) | ELPIDA_XSR ;
    MPMCDynRRD      = ( MPMCDynRRD & 0xfffffff0 ) | ELPIDA_RRD ;
    MPMCDynMRD      = ( MPMCDynMRD & 0xfffffff0 ) | ELPIDA_MRD ;

    MPMCDynCfg0     = ( MPMCDynCfg0 & 0xffffffff ) | 0x0 ;
    MPMCDynCfg1     = ( MPMCDynCfg1 & ELPIDA_MASK ) | ELPIDA_TYPE ;         /* buffers are disabled */
    MPMCDynCfg2     = ( MPMCDynCfg2 & ELPIDA_MASK ) | ELPIDA_TYPE ;
    MPMCDynCfg3     = ( MPMCDynCfg3 & 0xffffffff ) | 0x0 ;

    MPMCDynRasCas0  = ( MPMCDynRasCas0 & 0xffffffff ) | 0x0 ;
    MPMCDynRasCas1  = ( MPMCDynRasCas1 & 0xfffffcfc ) | ELPIDA_RASCAS ;
    MPMCDynRasCas2  = ( MPMCDynRasCas2 & 0xfffffcfc ) | ELPIDA_RASCAS ;
    MPMCDynRasCas3  = ( MPMCDynRasCas3 & 0xffffffff ) | 0x0 ;

    /* Issue MODE command */
    MPMCDynCtrl = ( MPMCDynCtrl & 0xffff1e58 ) | MPMC_MODE ;
    /* according to SDRAM data sheet no extra wait period is necessary */

    /* Program Elpida SDRAM mode register */
    read  = ELPIDA_READ;   /* A read access programs the mode register */

    /* Issue NORMAL command */
    MPMCDynCtrl = ( MPMCDynCtrl & 0xffff1e58 ) | MPMC_NORMAL ;
    /* according to SDRAM data sheet no extra wait period is necessary */

    /* Enable the buffers in the MPMCDynamicConfig registers */
    MPMCDynCfg0     = ( MPMCDynCfg0 & 0xffffffff ) | 0x0 ;
    MPMCDynCfg1     = ( MPMCDynCfg1 & ELPIDA_MASK ) | ELPIDA_OPERATION ;    /* buffers are enabled now, SDRAM ready for normal operation */
    MPMCDynCfg2     = ( MPMCDynCfg2 & ELPIDA_MASK ) | ELPIDA_OPERATION ;
    MPMCDynCfg3     = ( MPMCDynCfg3 & 0xffffffff ) | 0x0 ;
    /* according to SDRAM data sheet no extra wait period is necessary */
  #endif  /* INIT_SDRAM */
}

/****** EOF *********************************************************/
