/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

-------------------------------------------------------------------------
        File Information
  -------------------------------------------------------------------------
        Name    : SoCLite+.h
        Type    : C Programming Language File
        Date    : 2005 May 4th
        CPU     : ARM7TDMI-S  (SoCLite+)
        IDE     : IAR EWARM V4.31A
        Author  : NEC Electronics (Europe) GmbH                                           
  -------------------------------------------------------------------------
        About   : Header Function   (SoCLite+ Header)         
  -------------------------------------------------------------------------
        Copyright (C) NEC Corporation 2005
  -------------------------------------------------------------------------


 ****************************************************************************
 ****************************************************************************/

#ifndef SOCLITEPLUS_H  /* Avoid multiple inclusion */
#define SOCLITEPLUS_H

/******************************************************************************
 ******************************************************************************

          SoCLite+ System Register Definition

 ******************************************************************************
 ******************************************************************************/

struct bit
{
   unsigned int  p00: 1;
   unsigned int  p01: 1;
   unsigned int  p02: 1;
   unsigned int  p03: 1;
   unsigned int  p04: 1;
   unsigned int  p05: 1;
   unsigned int  p06: 1;
   unsigned int  p07: 1;
   unsigned int  p08: 1;
   unsigned int  p09: 1;
   unsigned int  p10: 1;
   unsigned int  p11: 1;
   unsigned int  p12: 1;
   unsigned int  p13: 1;
   unsigned int  p14: 1;
   unsigned int  p15: 1;
   unsigned int  p16: 1;
   unsigned int  p17: 1;
   unsigned int  p18: 1;
   unsigned int  p19: 1;
   unsigned int  p20: 1;
   unsigned int  p21: 1;
   unsigned int  p22: 1;
   unsigned int  p23: 1;
   unsigned int  p24: 1;
   unsigned int  p25: 1;
   unsigned int  p26: 1;
   unsigned int  p27: 1;
   unsigned int  p28: 1;
   unsigned int  p29: 1;
   unsigned int  p30: 1;
   unsigned int  p31: 1;
};



/*********************************************\
  MultiPort Memory Controller Registers
\*********************************************/

#define MPMC_BASE         (0x54003000)

#define MPMCCtrl          (*((volatile unsigned int *) (MPMC_BASE + 0x00)))    /* General Control */
#define MPMCStatus        (*((volatile unsigned int *) (MPMC_BASE + 0x04)))    /* General Status */
#define MPMCConf          (*((volatile unsigned int *) (MPMC_BASE + 0x08)))    /* General Configuration */
#define MPMCDynCtrl       (*((volatile unsigned int *) (MPMC_BASE + 0x20)))    /* Dynamic Control */
#define MPMCDynRef        (*((volatile unsigned int *) (MPMC_BASE + 0x24)))    /* Dynamic Refresh Timer */
#define MPMCDynReadConfig (*((volatile unsigned int *) (MPMC_BASE + 0x28)))    /* Dynamic Read Configuration - Strategy */
#define MPMCDynRP         (*((volatile unsigned int *) (MPMC_BASE + 0x30)))    /* Dynamic Precharge Period */
#define MPMCDynRAS        (*((volatile unsigned int *) (MPMC_BASE + 0x34)))    /* Dynamic RAS */
#define MPMCDynSREX       (*((volatile unsigned int *) (MPMC_BASE + 0x38)))    /* Dynamic Self-Refresh Time */
#define MPMCDynAPR        (*((volatile unsigned int *) (MPMC_BASE + 0x3C)))    /* Dynamic Last-Data-Out Time */
#define MPMCDynDAL        (*((volatile unsigned int *) (MPMC_BASE + 0x40)))    /* Dynamic Dat-In Time */
#define MPMCDynWR         (*((volatile unsigned int *) (MPMC_BASE + 0x44)))    /* Dynamic Write Recovery Time */
#define MPMCDynRC         (*((volatile unsigned int *) (MPMC_BASE + 0x48)))    /* Dynamic Active-to-Active */
#define MPMCDynRFC        (*((volatile unsigned int *) (MPMC_BASE + 0x4C)))    /* Dynamic Auto-Refresh */
#define MPMCDynXSR        (*((volatile unsigned int *) (MPMC_BASE + 0x50)))    /* Dynamic Exit Self-Refresh */
#define MPMCDynRRD        (*((volatile unsigned int *) (MPMC_BASE + 0x54)))    /* Dynamic Active Bank A-to-B */
#define MPMCDynMRD        (*((volatile unsigned int *) (MPMC_BASE + 0x58)))    /* Dynamic Load Mode */
#define MPMCStatExtWait   (*((volatile unsigned int *) (MPMC_BASE + 0x80)))    /* Static Extended Wait */
#define MPMCDynCfg0       (*((volatile unsigned int *) (MPMC_BASE + 0x100)))   /* Dynamic Configuration CS0 */
#define MPMCDynRasCas0    (*((volatile unsigned int *) (MPMC_BASE + 0x104)))   /* Dynamic RAS-CAS Delay CS0 */
#define MPMCDynCfg1       (*((volatile unsigned int *) (MPMC_BASE + 0x120)))   /* Dynamic Configuration CS1 */
#define MPMCDynRasCas1    (*((volatile unsigned int *) (MPMC_BASE + 0x124)))   /* Dynamic RAS-CAS Delay CS1 */
#define MPMCDynCfg2       (*((volatile unsigned int *) (MPMC_BASE + 0x140)))   /* Dynamic Configuration CS2 */
#define MPMCDynRasCas2    (*((volatile unsigned int *) (MPMC_BASE + 0x144)))   /* Dynamic RAS-CAS Delay CS2 */
#define MPMCDynCfg3       (*((volatile unsigned int *) (MPMC_BASE + 0x160)))   /* Dynamic Configuration CS3 */
#define MPMCDynRasCas3    (*((volatile unsigned int *) (MPMC_BASE + 0x164)))   /* Dynamic RAS-CAS Delay CS3 */
#define MPMCStatCfg0      (*((volatile unsigned int *) (MPMC_BASE + 0x200)))   /* Static Configuration CS0 */
#define MPMCStatWEWait0   (*((volatile unsigned int *) (MPMC_BASE + 0x204)))   /* Static Write-Enable Delay CS0 */
#define MPMCStatOEWait0   (*((volatile unsigned int *) (MPMC_BASE + 0x208)))   /* Static Output-Enable Delay CS0 */
#define MPMCStatRDWait0   (*((volatile unsigned int *) (MPMC_BASE + 0x20C)))   /* Static Read Delay CS0 */
#define MPMCStatPGWait0   (*((volatile unsigned int *) (MPMC_BASE + 0x210)))   /* Static Page-Mode Read CS0 */
#define MPMCStatWRWait0   (*((volatile unsigned int *) (MPMC_BASE + 0x214)))   /* Static Write Delay CS0 */
#define MPMCStatTRWait0   (*((volatile unsigned int *) (MPMC_BASE + 0x218)))   /* Static Turn-Round Delay CS0 */
#define MPMCStatCfg1      (*((volatile unsigned int *) (MPMC_BASE + 0x220)))   /* Static Configuration CS1 */
#define MPMCStatWEWait1   (*((volatile unsigned int *) (MPMC_BASE + 0x224)))   /* Static Write-Enable Delay CS1 */
#define MPMCStatOEWait1   (*((volatile unsigned int *) (MPMC_BASE + 0x228)))   /* Static Output-Enable Delay CS1 */
#define MPMCStatRDWait1   (*((volatile unsigned int *) (MPMC_BASE + 0x22C)))   /* Static Read Delay CS1 */
#define MPMCStatPGWait1   (*((volatile unsigned int *) (MPMC_BASE + 0x230)))   /* Static Page-Mode Read CS1 */
#define MPMCStatWRWait1   (*((volatile unsigned int *) (MPMC_BASE + 0x234)))   /* Static Write Delay CS1 */
#define MPMCStatTRWait1   (*((volatile unsigned int *) (MPMC_BASE + 0x238)))   /* Static Turn-Round Delay CS1 */
#define MPMCStatCfg2      (*((volatile unsigned int *) (MPMC_BASE + 0x240)))   /* Static Configuration CS2 */
#define MPMCStatWEWait2   (*((volatile unsigned int *) (MPMC_BASE + 0x244)))   /* Static Write-Enable Delay CS2 */
#define MPMCStatOEWait2   (*((volatile unsigned int *) (MPMC_BASE + 0x248)))   /* Static Output-Enable Delay CS2 */
#define MPMCStatRDWait2   (*((volatile unsigned int *) (MPMC_BASE + 0x24C)))   /* Static Read Delay CS2 */
#define MPMCStatPGWait2   (*((volatile unsigned int *) (MPMC_BASE + 0x250)))   /* Static Page-Mode Read CS2 */
#define MPMCStatWRWait2   (*((volatile unsigned int *) (MPMC_BASE + 0x254)))   /* Static Write Delay CS2 */
#define MPMCStatTRWait2   (*((volatile unsigned int *) (MPMC_BASE + 0x258)))   /* Static Turn-Round Delay CS2 */
#define MPMCStatCfg3      (*((volatile unsigned int *) (MPMC_BASE + 0x260)))   /* Static Configuration CS3 */
#define MPMCStatWEWait3   (*((volatile unsigned int *) (MPMC_BASE + 0x264)))   /* Static Write-Enable Delay CS3 */
#define MPMCStatOEWait3   (*((volatile unsigned int *) (MPMC_BASE + 0x268)))   /* Static Output-Enable Delay CS3 */
#define MPMCStatRDWait3   (*((volatile unsigned int *) (MPMC_BASE + 0x26C)))   /* Static Read Delay CS3 */
#define MPMCStatPGWait3   (*((volatile unsigned int *) (MPMC_BASE + 0x270)))   /* Static Page-Mode Read CS3 */
#define MPMCStatWRWait3   (*((volatile unsigned int *) (MPMC_BASE + 0x274)))   /* Static Write Delay CS3 */
#define MPMCStatTRWait3   (*((volatile unsigned int *) (MPMC_BASE + 0x278)))   /* Static Turn-Round Delay CS3 */
#define MPMCCITCR         (*((volatile unsigned int *) (MPMC_BASE + 0xF00)))   /* Test Control */
#define MPMCITIP          (*((volatile unsigned int *) (MPMC_BASE + 0xF20)))   /* Test Input */
#define MPMCITOP          (*((volatile unsigned int *) (MPMC_BASE + 0xF40)))   /* Test Output Register */
#define MPMCPerlD4        (*((volatile unsigned int *) (MPMC_BASE + 0xFD0)))   /* Peripheral ID bits 39:32 */
#define MPMCPerlD5        (*((volatile unsigned int *) (MPMC_BASE + 0xFD4)))   /* Peripheral ID (reserved) */
#define MPMCPerlD6        (*((volatile unsigned int *) (MPMC_BASE + 0xFD8)))   /* Peripheral ID (reserved) */
#define MPMCPerlD7        (*((volatile unsigned int *) (MPMC_BASE + 0xFDC)))   /* Peripheral ID (reserved) */
#define MPMCPerlD0        (*((volatile unsigned int *) (MPMC_BASE + 0xFE0)))   /* Peripheral ID bits 7:0 */
#define MPMCPerlD1        (*((volatile unsigned int *) (MPMC_BASE + 0xFE4)))   /* Peripheral ID bits 15:8 */
#define MPMCPerlD2        (*((volatile unsigned int *) (MPMC_BASE + 0xFE8)))   /* Peripheral ID bits 23:16 */
#define MPMCPerlD3        (*((volatile unsigned int *) (MPMC_BASE + 0xFEC)))   /* Peripheral ID bits 31:24 */
#define MPMCPCID0         (*((volatile unsigned int *) (MPMC_BASE + 0xFF0)))   /* PrimeCell ID bits 7:0 */
#define MPMCPCID1         (*((volatile unsigned int *) (MPMC_BASE + 0xFF4)))   /* PrimeCell ID bits 15:8 */
#define MPMCPCID2         (*((volatile unsigned int *) (MPMC_BASE + 0xFF8)))   /* PrimeCell ID bits 23:16 */ 
#define MPMCPCID3         (*((volatile unsigned int *) (MPMC_BASE + 0xFFC)))   /* PrimeCell ID bits 31:24 */ 

/********************************************************************/

#endif /* Avoid multiple inclusion */

/****** EOF *********************************************************/
