/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.60d                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : __low_level_init.c                (for NEC SoCLitePlus CPUs)
Purpose : Initializes SDRAM controller
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "SDRAM_Config.h"

/*********************************************************************
*
*       Configuration
*
*********************************************************************/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       __low_level_init()
*
*       Initialize memory controller, clock generation and pll
*
*       Has to be modified, if another CPU clock frequency should be
*       used. This function is called during startup and
*       has to return 1 to perform segment initialization
*
* NOTES:
*  (1) ICODE is the same segment as cstartup. By placing __low_level_init
*      in the same segment, we make sure it can be reached with BL.
*      Used in IAR environment only
*
*  (2) For IAR environment,
*      Return value has to be 1 to ensure segment initialization
*/
OS_INTERWORK int __low_level_init(void);  /* Avoid "no ptototype" warning */

#ifdef __IAR_SYSTEMS_ICC__
#pragma location="ICODE"       /* Note (1) */
#endif
OS_INTERWORK int __low_level_init(void) {
  SDRAM_Init();   /* Initialize and enable external SDRAM */
  //
  // Add more initialization here if required by application
  //
  return 1;       /* Note (2) */
}

/*****  EOF  ********************************************************/

